@@ -39,7 +39,7 @@ def execute_plan(
     sources,
     source_metadata,
     debug=False,
-    use_polars=True,
+    use_polars=False,
     first_match_only=False
 ):
     """
@@ -88,6 +88,10 @@ def execute_plan(
             # Can't inspect signature, assume no protocol
             return False
     
+    # Track whether we actually pushed the WHERE clause to the source
+    # If source doesn't support protocol, we need to keep pushable WHERE in plan.where_expr
+    where_was_pushed = False
+    
     # Apply optimizations if protocol is supported
     if source_supports_optimizations(root_source_fn) and (root_required_columns or pushable_where_sql):
         if debug:
@@ -103,6 +107,7 @@ def execute_plan(
             )
         
         root_source_fn = optimized_source_fn
+        where_was_pushed = bool(pushable_where_sql)  # Mark that WHERE was pushed
     
     # Start with scan of root table
     if debug:
@@ -121,9 +126,14 @@ def execute_plan(
     
     # Apply joins in order
     # Track WHERE clauses that have been pushed to sources
+    # IMPORTANT: Initialize remaining_where_expr from plan.where_expr AFTER we know
+    # whether pushable WHERE was pushed. If pushable WHERE wasn't pushed, we'll restore it later.
+    # For now, start with plan.where_expr (which might be None if optimizer moved everything to pushable_where_expr)
     remaining_where_expr = plan.where_expr
-    if debug and remaining_where_expr:
+    if debug:
         print(f"  [DEBUG] Initial remaining_where_expr: {remaining_where_expr}")
+        print(f"  [DEBUG] plan.pushable_where_expr: {plan.pushable_where_expr}")
+        print(f"  [DEBUG] where_was_pushed: {where_was_pushed}")
     
     for i, join_info in enumerate(plan.joins, 1):
         # Get required columns for joined table (needed for optimizations)
@@ -221,26 +231,108 @@ def execute_plan(
         )
     
     # Update plan.where_expr to only include clauses not pushed to sources
-    plan.where_expr = remaining_where_expr
+    # If pushable WHERE clause wasn't actually pushed (source doesn't support protocol),
+    # we need to keep it in plan.where_expr so it gets applied after scanning/joining
+    if not where_was_pushed and plan.pushable_where_expr:
+        # Source doesn't support protocol, so pushable WHERE wasn't pushed
+        # Combine pushable WHERE with remaining WHERE
+        from .optimizer import _combine_expressions
+        import sqlglot.expressions as exp
+        if remaining_where_expr:
+            plan.where_expr = _combine_expressions([plan.pushable_where_expr, remaining_where_expr], exp.And)
+        else:
+            plan.where_expr = plan.pushable_where_expr
+        if debug:
+            print(f"  [DEBUG] Restored pushable WHERE clause to plan.where_expr (wasn't pushed)")
+            print(f"  [DEBUG] Final plan.where_expr: {plan.where_expr}")
+    else:
+        # WHERE was pushed or there was no pushable WHERE, use remaining WHERE
+        plan.where_expr = remaining_where_expr
+        if debug:
+            print(f"  [DEBUG] Using remaining_where_expr: {plan.where_expr}")
     
     # Apply WHERE filter if present (non-pushable conditions)
     # Must be applied AFTER joins since remaining WHERE conditions may reference joined tables
     if plan.where_expr:
         if debug:
             print(f"  [FILTER] Applying WHERE clause (non-pushable conditions)")
+            print(f"  [DEBUG] WHERE expression type: {type(plan.where_expr)}")
+            print(f"  [DEBUG] WHERE expression: {plan.where_expr}")
+            if hasattr(plan.where_expr, 'this') and hasattr(plan.where_expr, 'expression'):
+                left = plan.where_expr.this
+                right = plan.where_expr.expression
+                print(f"  [DEBUG]   Left: {left} (type={type(left)})")
+                print(f"  [DEBUG]   Right: {right} (type={type(right)})")
+                if hasattr(right, 'this'):
+                    print(f"  [DEBUG]   Right value: {right.this} (type={type(right.this)})")
         
-        # Use Polars batch filtering if available and beneficial
-        if (use_polars and POLARS_AVAILABLE and PolarsBatchFilterIterator is not None):
+        # Apply WHERE filter
+        # IMPORTANT: For table-prefixed columns (like products.checked), we use Python FilterIterator
+        # to ensure correctness. Polars filtering can be unreliable with table-prefixed columns
+        # because the column names in the DataFrame might not match the filter expression.
+        # Python FilterIterator uses evaluate_expression() which correctly handles table prefixes.
+        
+        # Check if WHERE expression has table-prefixed columns
+        # IMPORTANT: Table-prefixed columns (like products.checked) require Python FilterIterator
+        # because Polars filtering has issues with table prefixes and type mismatches
+        # We check by looking at the left side of the equality expression
+        has_table_prefix = False
+        try:
+            from sqlglot import expressions as exp
+            # Simple check: if WHERE expression is an equality and left side is a Column with table prefix
+            if isinstance(plan.where_expr, exp.EQ):
+                left = plan.where_expr.this
+                if isinstance(left, exp.Column) and left.table:
+                    has_table_prefix = True
+                    if debug:
+                        print(f"  [DEBUG] Table prefix detected: {left.table}.{left.name}")
+            # Also check recursively for other expression types
+            elif hasattr(plan.where_expr, 'this'):
+                left = plan.where_expr.this
+                if isinstance(left, exp.Column) and left.table:
+                    has_table_prefix = True
+                    if debug:
+                        print(f"  [DEBUG] Table prefix detected in nested expression: {left.table}.{left.name}")
+            
+            if debug and not has_table_prefix:
+                print(f"  [DEBUG] No table prefix detected in WHERE expression")
+        except Exception as e:
+            # If check fails, assume it might have table prefixes (safer)
+            has_table_prefix = True
+            if debug:
+                print(f"  [DEBUG] Table prefix check failed: {e}, assuming True (safer)")
+        
+        # Use Polars filtering only if no table prefixes AND Polars is available
+        # IMPORTANT: Always use Python FilterIterator for table-prefixed columns to ensure correctness
+        # Polars filtering has issues with:
+        # 1. Table-prefixed column names (products.checked vs checked)
+        # 2. Type mismatches (string '1' vs int 1)
+        # Python FilterIterator handles both correctly
+        if (not has_table_prefix and use_polars and POLARS_AVAILABLE and PolarsBatchFilterIterator is not None):
+            if debug:
+                print(f"  [POLARS] Attempting Polars vectorized filtering (no table prefixes detected)")
             try:
                 iterator = PolarsBatchFilterIterator(iterator, plan.where_expr, batch_size=10000, debug=debug)
                 if debug:
-                    print(f"  [OPTIMIZATION] Using Polars vectorized filtering (SIMD)")
+                    print(f"  [POLARS] ✓ Polars filtering initialized successfully")
             except Exception as e:
                 if debug:
-                    print(f"  [OPTIMIZATION] Polars filtering failed: {e}, using Python")
+                    print(f"  [POLARS] ✗ Filtering failed: {e}, falling back to Python FilterIterator")
+                    import traceback
+                    traceback.print_exc()
+                # Fallback to Python filter - this MUST work
                 iterator = FilterIterator(iterator, plan.where_expr, debug=debug)
         else:
+            # Use Python filter iterator (more reliable for table-prefixed columns and type handling)
+            if debug:
+                if has_table_prefix:
+                    print(f"  [FILTER] Using Python FilterIterator (table-prefixed columns detected: products.checked)")
+                else:
+                    print(f"  [FILTER] Using Python FilterIterator (Polars not available or not requested)")
             iterator = FilterIterator(iterator, plan.where_expr, debug=debug)
+    else:
+        if debug:
+            print(f"  [DEBUG] No WHERE clause to apply (plan.where_expr is None)")
     
     # Apply projection
     if debug:
@@ -250,13 +342,13 @@ def execute_plan(
     
     # Use Polars batch projection if available and beneficial
     if (use_polars and POLARS_AVAILABLE and PolarsBatchProjectIterator is not None):
+        if debug:
+            print(f"  [POLARS] Using Polars vectorized projection (SIMD-accelerated)")
         try:
             iterator = PolarsBatchProjectIterator(iterator, plan.projections, batch_size=10000, debug=debug)
-            if debug:
-                print(f"  [OPTIMIZATION] Using Polars vectorized projection")
         except Exception as e:
             if debug:
-                print(f"  [OPTIMIZATION] Polars projection failed: {e}, using Python")
+                print(f"  [POLARS] ✗ Projection failed: {e}, using Python")
             iterator = ProjectIterator(iterator, plan.projections, debug=debug)
     else:
         iterator = ProjectIterator(iterator, plan.projections, debug=debug)
@@ -272,7 +364,7 @@ def _build_join_iterator(
     required_columns=None,
     optimized_right_source_fn=None,
     debug=False,
-    use_polars=True,
+    use_polars=False,
     first_match_only=False
 ):
     """
@@ -294,21 +386,35 @@ def _build_join_iterator(
     # For other sources, LookupJoinIterator will handle it via ScanIterator
     
     # Check if both sides are ordered by join keys
-    left_ordered_by = _extract_table_from_key(join_info.left_key)
+    # Extract left table name to check its metadata
+    left_table_name = _extract_table_from_key(join_info.left_key)
+    left_metadata = source_metadata.get(left_table_name, {}) if left_table_name else {}
+    left_ordered_by = left_metadata.get("ordered_by")
     right_ordered_by = right_metadata.get("ordered_by")
     
-    # For merge join, we need both sides sorted on their respective join keys
-    # This is a simplified check - in practice, we'd need to verify the actual
-    # column names match
+    # Extract column names from join keys
+    left_join_column = _extract_column_from_key(join_info.left_key)
+    right_join_column = _extract_column_from_key(join_info.right_key)
+    
+    # For merge join, we need BOTH sides sorted on their respective join keys
+    # Merge join REQUIRES sorted data - if data isn't sorted, it produces incorrect results
+    # Only use merge join when:
+    # 1. Explicitly told both sides are sorted (via ordered_by metadata)
+    # 2. NOT using Polars (Polars is faster for unsorted data, merge join only for sorted)
     use_merge_join = (
+        not use_polars and  # Don't use merge join if Polars is explicitly requested
+        left_ordered_by is not None and
+        left_ordered_by == left_join_column and
         right_ordered_by is not None and
-        right_ordered_by == _extract_column_from_key(join_info.right_key)
+        right_ordered_by == right_join_column
     )
     
     if use_merge_join:
         if debug:
             iterator_type = "MERGE JOIN"
             print(f"      Using {iterator_type} (sorted data)")
+            print(f"      Left ordered_by: {left_ordered_by}, Right ordered_by: {right_ordered_by}")
+            print(f"      Left join column: {left_join_column}, Right join column: {right_join_column}")
         return MergeJoinIterator(
             left_iterator,
             right_source,
@@ -320,13 +426,23 @@ def _build_join_iterator(
             debug=debug
         )
     else:
-        # Check if we can use mmap-based join (lowest memory - PRIORITY for large tables)
+        # When use_polars=True is explicitly set, prioritize Polars over mmap
+        # (unless user explicitly wants mmap by providing filename)
         right_metadata = source_metadata.get(join_info.table, {})
         right_table_filename = right_metadata.get("filename")
         
-        # For very large tables, prefer mmap over Polars (memory is critical)
-        if (MMAP_AVAILABLE and MmapLookupJoinIterator is not None and 
-            right_table_filename):
+        # Only use mmap if:
+        # 1. Filename is explicitly provided (user wants mmap)
+        # 2. Polars is NOT explicitly requested (use_polars=False)
+        # This ensures that when user sets use_polars=True, Polars is used instead of mmap
+        use_mmap = (
+            MMAP_AVAILABLE and 
+            MmapLookupJoinIterator is not None and 
+            right_table_filename and
+            not use_polars  # Don't use mmap if Polars is explicitly requested
+        )
+        
+        if use_mmap:
             if debug:
                 iterator_type = "MMAP LOOKUP JOIN"
                 if required_columns:
@@ -356,17 +472,35 @@ def _build_join_iterator(
                 pass
         
         # Decide between Polars and Python implementation
-        # NOTE: Only use Polars if mmap not available (mmap is better for memory)
-        if (use_polars and POLARS_AVAILABLE and 
-            should_use_polars(right_source, threshold=10000)):
+        # When use_polars=True, Polars is prioritized (unless mmap explicitly requested via filename)
+        # IMPORTANT: Call should_use_polars() only ONCE to avoid double I/O
+        should_use_polars_result = False
+        if use_polars and POLARS_AVAILABLE:
+            if debug:
+                print(f"      [POLARS] Checking if Polars should be used...")
+            try:
+                should_use_polars_result = should_use_polars(right_source, threshold=10000)
+                if debug:
+                    if not should_use_polars_result:
+                        print(f"      [POLARS] Right side too small or estimation failed, using Python join instead")
+            except Exception as e:
+                if debug:
+                    print(f"      [POLARS] Estimation failed: {e}, using Python join instead")
+                should_use_polars_result = False
+        elif debug:
+            if not use_polars:
+                print(f"      [POLARS] use_polars=False, using Python join")
+            elif not POLARS_AVAILABLE:
+                print(f"      [POLARS] Polars not available (not installed), falling back to Python")
+        
+        if (use_polars and POLARS_AVAILABLE and should_use_polars_result):
             if debug:
                 iterator_type = "POLARS LOOKUP JOIN"
+                print(f"      [POLARS] ✓ Using {iterator_type} (SIMD-accelerated)")
                 if required_columns:
-                    print(f"      Using {iterator_type} (fast, columns: {len(required_columns)})...")
-                else:
-                    print(f"      Using {iterator_type} (fast, vectorized)...")
+                    print(f"      [POLARS]   Columns: {len(required_columns)}")
                 if first_match_only:
-                    print(f"      ⚡ First-match-only mode enabled")
+                    print(f"      [POLARS]   ⚡ First-match-only mode enabled")
             try:
                 return PolarsLookupJoinIterator(
                     left_iterator,
@@ -383,7 +517,7 @@ def _build_join_iterator(
                 )
             except Exception as e:
                 if debug:
-                    print(f"      Polars join failed: {e}, falling back to Python")
+                    print(f"      [POLARS] ✗ Polars join failed: {e}, falling back to Python")
                 # Fallback to Python implementation
                 pass
         
