@@ -83,15 +83,63 @@ class FilterIterator:
     
     def __next__(self):
         while True:
-            row = next(self.source)
+            try:
+                row = next(self.source)
+            except StopIteration:
+                if self.debug and self._rows_processed == 0:
+                    print(f"      [DEBUG] FilterIterator: Source iterator is empty (no rows from upstream)")
+                raise
+            except Exception as e:
+                if self.debug:
+                    print(f"      [ERROR] FilterIterator: Failed to get row from source: {e}")
+                    import traceback
+                    traceback.print_exc()
+                raise
+            
             self._rows_processed += 1
             
-            if evaluate_expression(self.where_expr, row):
-                self._rows_passed += 1
-                if self.debug and self._rows_passed % 10000 == 0:
-                    print(f"      Filter: {self._rows_passed:,} passed / {self._rows_processed:,} processed")
-                return row
-            # Row filtered out - continue loop
+            try:
+                if self.debug and self._rows_processed <= 3:
+                    print(f"      [DEBUG] FilterIterator: Processing row {self._rows_processed}, keys={list(row.keys())[:5]}")
+                    print(f"      [DEBUG]   WHERE expression: {self.where_expr}")
+                    print(f"      [DEBUG]   WHERE expression type: {type(self.where_expr)}")
+                    if hasattr(self.where_expr, 'this'):
+                        left = self.where_expr.this
+                        print(f"      [DEBUG]   WHERE left side: {left} (type={type(left)})")
+                        if hasattr(left, 'name'):
+                            print(f"      [DEBUG]     Left column name: {left.name}, table: {getattr(left, 'table', None)}")
+                    if hasattr(self.where_expr, 'expression'):
+                        right = self.where_expr.expression
+                        print(f"      [DEBUG]   WHERE right side: {right} (type={type(right)})")
+                        if hasattr(right, 'this'):
+                            print(f"      [DEBUG]     Right value: {right.this} (type={type(right.this)})")
+                
+                result = evaluate_expression(self.where_expr, row)
+                
+                if self.debug and self._rows_processed <= 3:
+                    print(f"      [DEBUG] FilterIterator: Expression result={result} (type={type(result)})")
+                    if 'products.checked' in row:
+                        print(f"      [DEBUG]   products.checked={row.get('products.checked')} (type={type(row.get('products.checked'))})")
+                    elif 'checked' in row:
+                        print(f"      [DEBUG]   checked={row.get('checked')} (type={type(row.get('checked'))})")
+                
+                if result:
+                    self._rows_passed += 1
+                    if self.debug and self._rows_passed % 10000 == 0:
+                        print(f"      Filter: {self._rows_passed:,} passed / {self._rows_processed:,} processed")
+                    return row
+                # Row filtered out - continue loop
+                if self.debug and self._rows_processed <= 5:
+                    # Debug first few filtered rows
+                    print(f"      [DEBUG] Row filtered out: keys={list(row.keys())[:5]}, expr_result={result}")
+            except Exception as e:
+                if self.debug:
+                    print(f"      [ERROR] Filter evaluation failed: {e}")
+                    print(f"      [ERROR]   Row keys: {list(row.keys())[:10]}")
+                    import traceback
+                    traceback.print_exc()
+                # On error, skip row (conservative)
+                continue
 
 
 class ProjectIterator:
@@ -134,7 +182,10 @@ class ProjectIterator:
                     key = expr.name
                     result[key] = row[col_name]
                 else:
-                    raise KeyError(f"Column {col_name} not found in row")
+                    # Column not found - this can happen in LEFT JOIN when right side has no match
+                    # Return None (NULL) for missing columns (SQL standard behavior)
+                    key = expr.name
+                    result[key] = None
             else:
                 # Simple expression - use string representation as key
                 value = evaluate_expression(expr, row)
@@ -313,9 +364,9 @@ class LookupJoinIterator:
                     
                     self._join_count += 1
                     if self.debug and self._join_count % 10000 == 0:
-                        print(f"      Merge join: {self._join_count:,} rows matched")
+                        print(f"      Lookup join: {self._join_count:,} rows matched")
                     return {**left_row_copy, **right_row}
-            
+    
             # Handle LEFT JOIN
             else:  # LEFT JOIN
                 if not self._right_matches:
@@ -354,7 +405,7 @@ class LookupJoinIterator:
                     
                     self._join_count += 1
                     if self.debug and self._join_count % 10000 == 0:
-                        print(f"      Merge join: {self._join_count:,} rows matched")
+                        print(f"      Lookup join: {self._join_count:,} rows matched")
                     return {**left_row_copy, **right_row}
     
     def _get_key_value(self, row, key):
@@ -444,78 +495,81 @@ class MergeJoinIterator:
                 try:
                     self._left_row = next(self.left_source)
                 except StopIteration:
-                    # For LEFT JOIN, check if we have any remaining unmatched rows
-                    # (simplified - full implementation would track these)
                     raise StopIteration
             
             left_key_value = self._get_key_value(self._left_row, self.left_key)
             
-            # Advance right until we're at or past left key
-            while True:
-                if self._right_row is None:
-                    if not self._advance_right():
-                        # Right exhausted
-                        if self.join_type == "LEFT":
-                            # Yield left with NULLs
-                            result = self._left_row.copy()
-                            self._left_row = None
-                            return result
-                        # For INNER, skip remaining left rows
-                        raise StopIteration
+            # If we have a buffer with matching key and still have rows to process, use it
+            # This handles the case where we've already filled the buffer and are processing multiple matches
+            if (self._current_right_key == left_key_value and 
+                self._right_buffer_index < len(self._right_buffer)):
+                # Yield current combination from buffer
+                right_row = self._right_buffer[self._right_buffer_index]
+                self._right_buffer_index += 1
                 
-                right_key_value = self._get_key_value(self._right_row, self.right_key)
+                # Save left_row before potentially clearing it
+                result = {**self._left_row, **right_row}
                 
-                if right_key_value < left_key_value:
-                    # Right is behind, advance
-                    if not self._advance_right():
-                        # Right exhausted
-                        if self.join_type == "LEFT":
-                            result = self._left_row.copy()
-                            self._left_row = None
-                            return result
-                        raise StopIteration
-                    continue
-                elif right_key_value > left_key_value:
-                    # Right has passed, no match for this left
+                # If buffer exhausted, prepare for next left row
+                if self._right_buffer_index >= len(self._right_buffer):
+                    self._right_buffer_index = 0
+                    self._current_right_key = None  # Clear current key to force refill on next left row
+                    self._left_row = None
+                
+                self._join_count += 1
+                if self.debug and self._join_count % 10000 == 0:
+                    print(f"      Merge join: {self._join_count:,} rows matched")
+                return result
+            
+            # No buffer or buffer exhausted, need to advance right iterator
+            # Move right until right_key >= left_key
+            while self._right_row is None or \
+                self._get_key_value(self._right_row, self.right_key) < left_key_value:
+                if not self._advance_right():
                     if self.join_type == "LEFT":
                         result = self._left_row.copy()
                         self._left_row = None
                         return result
-                    # For INNER, skip this left row
+                    raise StopIteration
+            
+            right_key_value = self._get_key_value(self._right_row, self.right_key)
+            
+            if right_key_value > left_key_value:
+                if self.join_type == "LEFT":
+                    result = self._left_row.copy()
                     self._left_row = None
-                    break
-                else:
-                    # Keys match
-                    if self._current_right_key != left_key_value:
-                        # New key, fill buffer
-                        self._fill_right_buffer(left_key_value)
-                        self._current_right_key = left_key_value
-                    
-                    if not self._right_buffer:
-                        # No matches (shouldn't happen, but handle it)
-                        if self.join_type == "LEFT":
-                            result = self._left_row.copy()
-                            self._left_row = None
-                            return result
-                        self._left_row = None
-                        break
-                    
-                    # Yield current combination
-                    right_row = self._right_buffer[self._right_buffer_index]
-                    self._right_buffer_index += 1
-                    
-                    # Save left_row before potentially clearing it
-                    result = {**self._left_row, **right_row}
-                    
-                    # If we've exhausted right buffer, move to next left row
-                    if self._right_buffer_index >= len(self._right_buffer):
-                        self._right_buffer_index = 0
-                        self._left_row = None
-                    
-                    self._join_count += 1
-                    if self.debug and self._join_count % 10000 == 0:
-                        print(f"      Merge join: {self._join_count:,} rows matched")
                     return result
+                self._left_row = None
+                continue
+            
+            # right_key == left_key → buffer all matches
+            self._fill_right_buffer(left_key_value)
+            self._current_right_key = left_key_value
+            self._right_buffer_index = 0  # Reset buffer index for new key
+            
+            if not self._right_buffer:
+                if self.join_type == "LEFT":
+                    result = self._left_row.copy()
+                    self._left_row = None
+                    return result
+                self._left_row = None
+                continue
+            
+            # Yield first combination from buffer
+            right_row = self._right_buffer[self._right_buffer_index]
+            self._right_buffer_index += 1
+            
+            result = {**self._left_row, **right_row}
+            
+            # If buffer exhausted, prepare for next left row
+            if self._right_buffer_index >= len(self._right_buffer):
+                self._right_buffer_index = 0
+                self._left_row = None
+            
+            self._join_count += 1
+            if self.debug and self._join_count % 10000 == 0:
+                print(f"      Merge join: {self._join_count:,} rows matched")
+            return result
     
     def _get_key_value(self, row, key):
         """Extract join key value from a row."""
