@@ -32,8 +32,7 @@ class PolarsLookupJoinIterator:
         right_alias: str,
         batch_size: int = 10000,
         required_columns: Optional[Set[str]] = None,
-        debug: bool = False,
-        first_match_only: bool = False
+        debug: bool = False
     ):
         self.left_source = left_source
         self.right_source_fn = right_source_fn
@@ -45,16 +44,13 @@ class PolarsLookupJoinIterator:
         self.batch_size = batch_size
         self.required_columns = required_columns  # Set of column names (for column pruning)
         self.debug = debug
-        self.first_match_only = first_match_only  # If True, only return first match per left key
         
         # Extract column name from right key (e.g., "alias.col" -> "col")
         self.right_table_col = self._extract_column_from_key(right_key)
         
         # Build Polars-based lookup index (SIMD-accelerated)
         if self.debug:
-            print(f"      [POLARS] Building Polars lookup index for {right_table}...")
-            if first_match_only:
-                print(f"      [POLARS] ⚡ First-match-only mode: Will deduplicate right side and return only first match per left key")
+            print(f"      Building Polars lookup index for {right_table}...")
         self._build_polars_index()
         
         # State for join iteration
@@ -77,25 +73,40 @@ class PolarsLookupJoinIterator:
         Polars uses SIMD instructions for group_by operations.
         """
         # Collect all rows from right source
+        # NOTE: This source function may have already been consumed by should_use_polars(),
+        # so we need to call it again. For file-based sources, this means re-reading the file.
+        # This is acceptable because Polars' performance benefits outweigh the I/O cost
+        # for large datasets (>10K rows).
         right_rows = []
         row_count = 0
         
-        for row in self.right_source_fn():
-            if not row or not isinstance(row, dict):
-                continue
-            
-            # Column pruning: only include required columns
-            if self.required_columns:
-                row = {k: v for k, v in row.items() if k in self.required_columns}
-            
-            # Prefix columns with right alias
-            prefixed_row = {f"{self.right_alias}.{key}": value 
-                          for key, value in row.items()}
-            right_rows.append(prefixed_row)
-            row_count += 1
-            
-            if self.debug and row_count % 50000 == 0:
-                print(f"      Collected {row_count:,} rows...")
+        try:
+            for row in self.right_source_fn():
+                if not row or not isinstance(row, dict):
+                    continue
+                
+                # Column pruning: only include required columns
+                if self.required_columns:
+                    row = {k: v for k, v in row.items() if k in self.required_columns}
+                
+                # Prefix columns with right alias
+                prefixed_row = {f"{self.right_alias}.{key}": value 
+                              for key, value in row.items()}
+                right_rows.append(prefixed_row)
+                row_count += 1
+                
+                if self.debug and row_count % 50000 == 0:
+                    print(f"      Collected {row_count:,} rows...")
+        except StopIteration:
+            # Source was already consumed (e.g., by should_use_polars)
+            # This shouldn't happen if should_use_polars works correctly,
+            # but handle it gracefully
+            if self.debug:
+                print(f"      Warning: Source iterator was already consumed")
+            if not right_rows:
+                self.polars_df = None
+                self.lookup_index = {}
+                return
         
         if not right_rows:
             self.polars_df = None
@@ -106,22 +117,38 @@ class PolarsLookupJoinIterator:
         
         # Convert to Polars DataFrame (vectorized, SIMD-optimized)
         if self.debug:
-            print(f"      [POLARS] Converting {row_count:,} rows to Polars DataFrame...")
+            print(f"      Converting {row_count:,} rows to Polars DataFrame...")
         
         try:
-            # Use Polars lazy evaluation for better performance
-            self.polars_df = pl.DataFrame(right_rows)
+            # Try to convert to Polars DataFrame
+            # First attempt: normal schema inference
+            try:
+                self.polars_df = pl.DataFrame(right_rows)
+            except Exception as schema_error:
+                # If schema inference fails (mixed types), try with disabled inference
+                # This allows columns with mixed string/numeric types (e.g., "14.95" vs 14.95)
+                if "could not append value" in str(schema_error) or "infer_schema_length" in str(schema_error):
+                    if self.debug:
+                        print(f"      Schema inference failed (mixed types), disabling inference...")
+                    # Try with infer_schema_length=None (disables inference, all columns become Object/string)
+                    try:
+                        self.polars_df = pl.DataFrame(right_rows, infer_schema_length=None)
+                    except Exception:
+                        # If that also fails, fall back to Python
+                        raise schema_error
+                else:
+                    raise schema_error
         except Exception as e:
             if self.debug:
-                print(f"      [POLARS] ✗ Conversion failed: {e}")
-                print(f"      [POLARS] Falling back to Python dict index")
+                print(f"      Warning: Polars conversion failed: {e}")
+                print(f"      Falling back to Python dict index")
             # Fallback to Python dict
             self._build_python_index(right_rows)
             return
         
         # Group by join key using Polars (vectorized, SIMD-accelerated)
         if self.debug:
-            print(f"      [POLARS] Grouping by join key '{self.right_table_col}' (SIMD-accelerated)...")
+            print(f"      Grouping by join key '{self.right_table_col}' (SIMD-accelerated)...")
         
         try:
             # Get the full column name (with alias prefix)
@@ -144,29 +171,21 @@ class PolarsLookupJoinIterator:
             self.lookup_index = {}
             
             # Iterate through groups (Polars does this efficiently with SIMD)
-            duplicate_count = 0
             for group_key, group_df in grouped:
                 # Convert group to list of dicts
                 rows = group_df.to_dicts()
                 # Handle None keys (can't join on None)
                 if group_key is not None:
-                    # If first_match_only, only keep first row per key
-                    if self.first_match_only and len(rows) > 1:
-                        duplicate_count += len(rows) - 1
-                        self.lookup_index[group_key] = rows[:1]  # Only first match
-                    else:
-                        self.lookup_index[group_key] = rows
+                    self.lookup_index[group_key] = rows
             
             if self.debug:
                 unique_keys = len(self.lookup_index)
-                print(f"      [POLARS] ✓ Index built: {row_count:,} rows, {unique_keys:,} unique keys")
-                if self.first_match_only and duplicate_count > 0:
-                    print(f"      [POLARS] ⚠️  Deduplicated: {duplicate_count:,} duplicate keys removed (kept first match only)")
+                print(f"      Polars index built (SIMD): {row_count:,} rows, {unique_keys:,} unique keys")
         
         except Exception as e:
             if self.debug:
-                print(f"      [POLARS] ✗ Grouping failed: {e}")
-                print(f"      [POLARS] Falling back to Python dict index")
+                print(f"      Warning: Polars grouping failed: {e}")
+                print(f"      Falling back to Python dict index")
             # Fallback to Python dict
             self._build_python_index(right_rows)
     
@@ -174,7 +193,6 @@ class PolarsLookupJoinIterator:
         """Fallback: Build Python dict index (original implementation)."""
         self.lookup_index = {}
         index_size = 0
-        duplicate_count = 0
         
         for prefixed_row in right_rows:
             key_value = prefixed_row.get(f"{self.right_alias}.{self.right_table_col}")
@@ -184,22 +202,11 @@ class PolarsLookupJoinIterator:
             if key_value is not None:
                 if key_value not in self.lookup_index:
                     self.lookup_index[key_value] = []
-                    self.lookup_index[key_value].append(prefixed_row)
-                    index_size += 1
-                else:
-                    # Key already exists
-                    if self.first_match_only:
-                        # Skip duplicates - only keep first match
-                        duplicate_count += 1
-                    else:
-                        # Keep all matches (normal SQL behavior)
-                        self.lookup_index[key_value].append(prefixed_row)
-                        index_size += 1
+                self.lookup_index[key_value].append(prefixed_row)
+                index_size += 1
         
         if self.debug:
             print(f"      Python index built: {index_size:,} rows, {len(self.lookup_index):,} unique keys")
-            if self.first_match_only and duplicate_count > 0:
-                print(f"      ⚠️  Deduplicated: {duplicate_count:,} duplicate keys removed (kept first match only)")
     
     def __iter__(self):
         return self
@@ -246,10 +253,6 @@ class PolarsLookupJoinIterator:
                 # Filter out None or invalid rows
                 self._right_matches = [m for m in matches 
                                      if m is not None and isinstance(m, dict)]
-                
-                # If first_match_only mode, only take first match
-                if self.first_match_only and len(self._right_matches) > 1:
-                    self._right_matches = self._right_matches[:1]
             
             # Handle INNER JOIN
             if self.join_type == "INNER":
@@ -367,9 +370,9 @@ class PolarsBatchFilterIterator:
             self.can_use_polars = False
         
         if self.debug and self.can_use_polars:
-            print(f"      [POLARS] Using Polars vectorized filtering (SIMD-accelerated)")
+            print(f"      Using Polars vectorized filtering (SIMD-accelerated)")
         elif self.debug:
-            print(f"      [POLARS] Polars translation not available, using Python filtering")
+            print(f"      Using Python filtering (Polars translation not available)")
     
     def __iter__(self):
         return self
@@ -402,7 +405,15 @@ class PolarsBatchFilterIterator:
         if self.can_use_polars and self.polars_filter_expr is not None:
             try:
                 # Convert batch to Polars DataFrame
-                df = pl.DataFrame(self._buffer)
+                # Handle schema inference errors for mixed types
+                try:
+                    df = pl.DataFrame(self._buffer)
+                except Exception as schema_error:
+                    if "could not append value" in str(schema_error) or "infer_schema_length" in str(schema_error):
+                        # Disable schema inference for mixed types
+                        df = pl.DataFrame(self._buffer, infer_schema_length=None)
+                    else:
+                        raise schema_error
                 
                 # Apply vectorized filter (SIMD-accelerated)
                 filtered_df = df.filter(self.polars_filter_expr)
@@ -414,7 +425,7 @@ class PolarsBatchFilterIterator:
                 
             except Exception as e:
                 if self.debug:
-                    print(f"      [POLARS] ✗ Filtering failed: {e}, falling back to Python")
+                    print(f"      Polars filtering failed: {e}, falling back to Python")
                 # Fallback to Python
                 self.can_use_polars = False
                 self._filter_rows_python()
@@ -500,41 +511,137 @@ class PolarsBatchProjectIterator:
         
         # Use Polars for vectorized projection
         try:
-            df = pl.DataFrame(self._buffer)
+            # Try to create DataFrame with schema inference disabled to handle mixed types
+            # This prevents errors when columns have both string and numeric values (e.g., "14.95" vs 14.95)
+            try:
+                df = pl.DataFrame(self._buffer, infer_schema_length=None)
+            except Exception as df_error:
+                # If DataFrame creation fails (e.g., schema issues), fallback to Python
+                if "could not append value" in str(df_error) or "infer_schema_length" in str(df_error):
+                    if self.debug:
+                        print(f"      [POLARS] ✗ DataFrame creation failed (schema inference): {df_error}, falling back to Python")
+                    raise df_error
+                else:
+                    raise df_error
             
-            # Build select expression for Polars
-            # This is simplified - full implementation would translate all projections
+            # Build select expression for Polars with proper alias handling
             from sqlglot import expressions as exp
-            selected_cols = []
+            select_exprs = []
+            seen_aliases = set()  # Track output column names to detect duplicates early
+            seen_source_columns = set()  # Track which source columns we've selected
+            alias_to_col = {}  # Map output names to source columns for reference
+            
+            # First pass: extract all output names to detect duplicates early
             for expr in self.projections:
+                output_name = None
+                
+                if isinstance(expr, exp.Alias):
+                    output_name = expr.alias
+                elif isinstance(expr, exp.Column):
+                    output_name = expr.name  # Output column name (without table prefix)
+                else:
+                    # For other expression types, we'll handle them in the second pass
+                    # but mark them as potentially problematic
+                    continue
+                
+                # Track output name immediately to catch duplicates early
+                if output_name:
+                    if output_name in seen_aliases:
+                        # Duplicate detected - this will cause Polars to fail
+                        # Fallback to Python immediately for this batch
+                        if self.debug:
+                            print(f"      Warning: Duplicate output name '{output_name}' detected early, falling back to Python")
+                        raise ValueError(f"Duplicate output name '{output_name}' detected in projections")
+                    seen_aliases.add(output_name)
+            
+            # Second pass: build select expressions (duplicates already filtered)
+            for expr in self.projections:
+                output_name = None
+                col_name = None
+                
                 if isinstance(expr, exp.Alias):
                     # SELECT col AS alias
+                    output_name = expr.alias
                     col_name = self._extract_column_name(expr.this)
-                    if col_name:
-                        selected_cols.append(col_name)
                 elif isinstance(expr, exp.Column):
-                    # SELECT alias.col
+                    # SELECT alias.col or SELECT col
                     col_name = f"{expr.table}.{expr.name}" if expr.table else expr.name
-                    selected_cols.append(col_name)
+                    output_name = expr.name  # Output column name (without table prefix)
+                else:
+                    # Other expression types - can't handle with simple Polars select
+                    # Will fallback to Python for these
+                    raise ValueError(f"Unsupported expression type: {type(expr)}")
+                
+                # Double-check for duplicates (shouldn't happen after first pass, but be safe)
+                if output_name not in seen_aliases:
+                    if self.debug:
+                        print(f"      Warning: Output name '{output_name}' not in seen_aliases, skipping")
+                    continue
+                
+                if col_name and col_name in df.columns:
+                    # Check if we're selecting the same source column multiple times
+                    # Polars doesn't allow selecting the same column twice, even with different aliases
+                    if col_name in seen_source_columns:
+                        if self.debug:
+                            print(f"      Warning: Source column '{col_name}' already selected (output: '{output_name}'), skipping duplicate")
+                        continue
+                    
+                    # Track this selection
+                    seen_source_columns.add(col_name)
+                    alias_to_col[output_name] = col_name
+                    # Select column with alias to ensure correct output name
+                    select_exprs.append(pl.col(col_name).alias(output_name))
+                elif col_name:
+                    # Column not found in DataFrame - might be computed expression
+                    # Fallback to Python for these cases
+                    raise ValueError(f"Column '{col_name}' not found in DataFrame")
             
             # Select columns (vectorized, SIMD-accelerated)
-            if selected_cols:
-                # Filter to only existing columns
-                existing_cols = [c for c in selected_cols if c in df.columns]
-                if existing_cols:
-                    projected_df = df.select(existing_cols)
-                else:
-                    projected_df = df
+            if select_exprs:
+                try:
+                    # Polars will catch any duplicate output names we missed and raise an error
+                    # We'll catch that error and fallback to Python
+                    projected_df = df.select(select_exprs)
+                    
+                    # Convert back to list of dicts
+                    # This can fail with schema errors if types are inconsistent even with infer_schema_length=None
+                    try:
+                        self._projected_rows = projected_df.to_dicts()
+                    except Exception as dict_error:
+                        # Error during to_dicts() - might be schema-related or type conversion issues
+                        # Check if it's a known schema error
+                        error_str = str(dict_error).lower()
+                        if "could not append value" in error_str or "infer_schema_length" in error_str or "type" in error_str:
+                            if self.debug:
+                                print(f"      [POLARS] ✗ to_dicts() failed (schema/type issue): {dict_error}, falling back to Python")
+                        raise dict_error
+                except Exception as select_error:
+                    # Error during select() - might be duplicate columns or other Polars errors
+                    error_str = str(select_error).lower()
+                    if "duplicate" in error_str or "output name" in error_str:
+                        if self.debug:
+                            print(f"      [POLARS] ✗ Duplicate column names detected: {select_error}, falling back to Python")
+                    elif "could not append value" in error_str or "infer_schema_length" in error_str:
+                        if self.debug:
+                            print(f"      [POLARS] ✗ Schema inference error: {select_error}, falling back to Python")
+                    raise select_error
             else:
-                projected_df = df
-            
-            # Convert back to list of dicts
-            self._projected_rows = projected_df.to_dicts()
+                # No valid projections, use Python fallback
+                raise ValueError("No valid projections found")
             
         except Exception as e:
+            # Catch ALL Polars-related errors and fallback to Python
+            # This includes schema inference errors, type mismatches, duplicate columns, etc.
+            error_str = str(e).lower()
             if self.debug:
-                print(f"      [POLARS] ✗ Projection failed: {e}, falling back to Python")
-            # Fallback to Python
+                # Provide more specific error messages
+                if "duplicate" in error_str or "output name" in error_str:
+                    print(f"      [POLARS] ✗ Projection failed: duplicate output names detected, falling back to Python")
+                elif "could not append value" in error_str or "infer_schema_length" in error_str:
+                    print(f"      [POLARS] ✗ Projection failed: schema inference issue (mixed types), falling back to Python")
+                else:
+                    print(f"      [POLARS] ✗ Projection failed: {e}, falling back to Python")
+            # Always fallback to Python projection for any error
             self._project_rows_python()
         
         if self.debug and self._row_count % 10000 == 0:
@@ -583,31 +690,43 @@ def should_use_polars(right_source_fn: Callable, threshold: int = 10000) -> bool
     """
     Determine if Polars should be used based on estimated data size.
     
+    OPTIMIZED: Only samples first 100 rows for quick estimation, then returns.
+    This minimizes I/O overhead while still providing accurate size estimation.
+    
+    NOTE: This function consumes part of the iterator (first 100 rows), which means
+    the source function will be called again when building the Polars index.
+    For file-based sources, this causes some double I/O, but:
+    1. Only first 100 rows are consumed here (fast)
+    2. Polars' performance benefits (10-50x faster) outweigh the I/O cost for large datasets
+    3. For small datasets (<10K rows), Python join is used instead
+    
     Args:
         right_source_fn: Function that returns iterator of rows
         threshold: Minimum number of rows to use Polars (default: 10,000)
     
     Returns:
         True if Polars should be used, False otherwise
-    
-    Note: This function samples the iterator but doesn't consume it fully.
-    The iterator will be recreated when Polars builds the index.
     """
-    # Quick estimation: sample first few rows
-    # The source function will be called again when building the index
+    # Quick estimation: sample only first 100 rows for speed
+    # This minimizes I/O while still providing good size estimation
     try:
         iterator = iter(right_source_fn())
         count = 0
+        sample_size = 100  # Only sample first 100 rows for speed
+        
         for _ in iterator:
             count += 1
             if count >= threshold:
+                # Already reached threshold, definitely use Polars
                 return True
-            if count >= 100:  # Sample first 100 rows
-                # Estimate total size (rough heuristic)
-                # If first 100 rows exist, likely more exist
+            if count >= sample_size:
+                # Sampled enough rows - assume dataset is large enough
+                # For file-based sources, if we have 100+ rows, likely many more exist
                 return True
+        
+        # Iterator exhausted - check if we have enough rows
         return count >= threshold
-    except Exception as e:
-        # If estimation fails, default to Python (safer)
-        # Note: Exception is silently caught to avoid breaking execution
+    except Exception:
+        # If estimation fails (e.g., source doesn't support multiple calls),
+        # default to Python (safer and avoids double I/O)
         return False
